<?php
namespace FiloBlu\Rma\Observer;
use FiloBlu\Rma\Helper\ChangeSizeAttributeHelper;
use FiloBlu\Rma\Helper\ChangeSizeOrder;
use FiloBlu\Rma\Logger\Logger;
use Magento\Framework\Event\Observer;
use Magento\Framework\Event\ObserverInterface;
use Magento\Framework\Exception\LocalizedException;
use Magento\Sales\Api\OrderRepositoryInterface;

class UpdateStatusVerify implements ObserverInterface
{
    /**
     * @var Logger
     */
    private $logger;
    /**
     * @var ChangeSizeOrder
     */
    private $changeSizeOrder;
    /**
     * @var OrderRepositoryInterface
     */
    private $orderRepository;


    /**
     * UpdateStatusVerify constructor.
     * @param Logger $logger
     * @param ChangeSizeOrder $changeSizeOrder
     * @param OrderRepositoryInterface $orderRepository
     */
    public function __construct(
        Logger $logger,
        ChangeSizeOrder $changeSizeOrder,
        OrderRepositoryInterface $orderRepository){
        $this->logger = $logger;
        $this->changeSizeOrder = $changeSizeOrder;
        $this->orderRepository = $orderRepository;
    }


    /**
     * @param Observer $observer
     * @throws LocalizedException
     */
    public function execute(Observer $observer)
    {
        $order = $observer->getEvent()->getOrder();
        if ($order && $order->getId() && $this->changeSizeOrder->isChangeSizeOrder($order->getId())) {
            $toStatus = $order->getData("status");
            $fromStatus = $order->getOrigData("status");
            $allowedStatus = ["order_exported", ChangeSizeAttributeHelper::ORDER_STATUS_CHANGESIZE_CODE,"pending","canceled","confirmed","confirmed_by_ext","payment_confirmed","payment_confirmed_on_ext"];
            if (!in_array($toStatus, $allowedStatus)) {
                if (!$this->changeSizeOrder->isValidForUpdateStatus($order->getId())) {
                    $this->logger->debug("FiloBlu\Rma\Observer: Order Status cannot be updated before linked RMA is processed .Order: " . $order->getIncrementId() . " is restored in Status Order Exported");
                    if (!in_array($fromStatus, $allowedStatus)) {
                        $this->logger->debug("FiloBlu\Rma\Observer: Order " . $order->getIncrementId() . " is restored to Order Exported Status.");
                        $order->setStatus('order_exported');
                        $order->setState('processing');
                        $order->setChangeSizeRestored(true);
                        $order->addStatusHistoryComment("Order is restored to Order Exported programmatically because linked RMA is still processing.");
                        $this->orderRepository->save($order);
                    }
                    if(!$order->getChangeSizeRestored()){
                        throw new LocalizedException(__("Status update not allowed!"));
                    }
                }
            }
        }
    }

}